==> org-lsst-ccs-subsystem-demo-main-5.1.0-sources.jar: org/lsst/ccs/subsystem/demo/main/DemoSubsystem.java <==
package org.lsst.ccs.subsystem.demo.main;

import java.util.concurrent.TimeUnit;
import org.lsst.ccs.bus.data.KeyValueData;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.subsystem.demo.bus.DemoData;
import org.lsst.ccs.utilities.scheduler.PeriodicTask;
import hep.aida.*;
import hep.aida.util.XMLUtils;
import java.io.IOException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.Random;
import org.lsst.ccs.Subsystem;
import org.lsst.ccs.bus.data.Alert;
import org.lsst.ccs.bus.data.KeyValueDataList;
import org.lsst.ccs.bus.states.AlertState;
import org.lsst.ccs.command.annotations.Command.CommandType;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.framework.AgentPeriodicTask;
import org.lsst.ccs.framework.HasLifecycle;
import org.lsst.ccs.services.AgentPeriodicTaskService;
import org.lsst.ccs.utilities.scheduler.Scheduler;

public class DemoSubsystem implements HasLifecycle {
    
    public static enum DemoState {
        IDLE, BUSY;
    }
    
    public static enum OtherState {
        LOW, HIGH;
    }
    
    public static enum AState {
        A,B;
    }

    /**
     * Scheduler task that publishes heartbeat status messages.
     */
    protected PeriodicTask dataPublisher;
    protected PeriodicTask plotPublisher;

    //Create analysis factory
    private final IAnalysisFactory af = IAnalysisFactory.create();
    // Create datapoint set factory
    private final IDataPointSetFactory dataPointSetFactory = af.createDataPointSetFactory(null);
    // Create histogram factory
    private final IHistogramFactory histogramFactory = af.createHistogramFactory(null);

    private final Random r = new Random();

    private final ArrayList<PeriodicTask> tracerTestTasks = new ArrayList<>();

    @LookupField(strategy = LookupField.Strategy.TREE)
    protected AgentPeriodicTaskService periodicTaskService;

    @LookupField(strategy = LookupField.Strategy.TOP)
    protected Subsystem subsys;
    
    private int dataPublishRate = 5;
    private int dataSize = 2;
    
    /**
     ***************************************************************************
     **
     ** Initializes the subsystem. *
     * **************************************************************************
     */
    @Override
    public void postStart() {
        // This must not be invoked in init because the subsystem is not yet 
        // connected on the buses.
        updateLimits("1.0", "2.0");
    }

    @Override
    public void build() {

        //Schedule a periodic task to publish demo data
        periodicTaskService.scheduleAgentPeriodicTask(
                new AgentPeriodicTask("demoData-publish",
                        () -> {
                            DemoData dd = new DemoData(dataSize);
                            KeyValueData d = new KeyValueData("demo_Data", dd);
                            KeyValueDataList kvdl = new KeyValueDataList();
                            kvdl.addData(d);
                            subsys.publishSubsystemDataOnStatusBus(kvdl);
                        }).withIsFixedRate(false).withPeriod(Duration.ofSeconds(dataPublishRate)));
        
        //Schedule a periodic task to publish demo plots
        periodicTaskService.scheduleAgentPeriodicTask(
                new AgentPeriodicTask("demoPlots-publish",
                        () -> {
                            try {
                                
                                KeyValueDataList plots = new KeyValueDataList("");
                                IHistogram1D histObj = histogramFactory.createHistogram1D("test 1d", 50, -3, 6);
                                for (int i = 0; i < 500; i++) {
                                    histObj.fill(r.nextGaussian());
                                }
                                
                                IDataPointSet dps1D = dataPointSetFactory.create("dps1D", "one dimensional IDataPointSet", 1);
                                for (int i = 0; i < 10; i++) {
                                    dps1D.addPoint().coordinate(0).setValue(r.nextDouble());
                                }
                                
                                plots.addData("histogram1D", XMLUtils.createXMLString((IManagedObject) histObj), KeyValueData.KeyValueDataType.KeyValuePlotData);
                                plots.addData("dataPointSet", XMLUtils.createXMLString((IManagedObject) dps1D), KeyValueData.KeyValueDataType.KeyValuePlotData);
                                
                                subsys.publishSubsystemDataOnStatusBus(plots);
                            } catch (IOException ioe) {
                                ioe.printStackTrace();
                            }
                        }).withIsFixedRate(false).withPeriod(Duration.ofSeconds(5)));        
        
    }
    
    @Command(type = CommandType.ACTION, description = "update limits")
    public void updateLimits(String low, String high) {
        KeyValueDataList limits = new KeyValueDataList("demo_Data/temp");
        limits.addData("alarmLow", low, KeyValueData.KeyValueDataType.KeyValueMetaData);
        limits.addData("alarmHigh", high, KeyValueData.KeyValueDataType.KeyValueMetaData);
        subsys.publishSubsystemDataOnStatusBus(limits);
    }

    @Command(type = CommandType.QUERY, description = "raise custom alert")
    public void raiseCustomAlert(String alertId, AlertState severity, String cause) {
        Alert a = new Alert(alertId, "Custom Alert");
        subsys.getAlertService().raiseAlert(a, severity, cause);
    }
    
    @Command(type = CommandType.QUERY, description = "Get Subsystem Data Sample")
    public DemoData getSampleData() {
        return new DemoData(dataSize);
    }

    @Command(type = CommandType.QUERY, description = "Set Subsystem Data Sample")
    public void setSampleDataSize(int size) {
        this.dataSize = size;
    }
    
    
    @Command(type = CommandType.QUERY, description = "Get Subsystem Array Sample")
    public Object[] getSampleArrayData() {
        Object[] data = new Object[]{"test", 2.3, new int[]{3, 2, 2, 2}};
        return data;
    }
    
    @Command(type = CommandType.QUERY, description = "Throw an Exception")
    public void throwAnException() {
        throw new RuntimeException("An Exception");
    }

    @Command(type = CommandType.ACTION, description = "Enable/Disable Publishing Tracer Test Messages")
    public void runTracerTest(boolean enable) {
        if (enable) {
            Scheduler s = subsys.getScheduler();
            tracerTestTasks.add(s.scheduleAtFixedRate(() -> subsys.getLogger().info("Self test on module " + r.nextInt(1000) + (r.nextInt(5) > 0 ? ": OK" : ": FAILED")), 0, 7, TimeUnit.SECONDS));
            tracerTestTasks.add(s.scheduleAtFixedRate(() -> subsys.getLogger().warn("London Bridge is in trouble, please repair."), 2, 15, TimeUnit.SECONDS));
            tracerTestTasks.add(s.scheduleAtFixedRate(() -> subsys.getLogger().error("Sun has gone supernova."), 3, 13, TimeUnit.SECONDS));
        } else {
            tracerTestTasks.forEach(task -> task.cancel(false));
            tracerTestTasks.clear();
        }
    }

    @Command(type = CommandType.QUERY, description = "Time out")
    public String testTimeout(int sleep) {
        if (sleep > 0) {
            try {
                Thread.sleep(sleep * 1000L);
            } catch (InterruptedException x) {
                return "Interrupted";
            }
        }
        return "Done.";
    }

    @Command(type = CommandType.QUERY, description = "Test sending ACK/NACK. Delays are in seconds.", autoAck = false)
    public String testAck(boolean accept, int delayBeforeAck, int delayAfterAck, int timeout) {

        if (delayBeforeAck > 0) {
            try {
                Thread.sleep(delayBeforeAck * 1000L);
            } catch (InterruptedException x) {
                return "Interrupted while waiting to send ACK/NACK";
            }
        }

        if (accept) {
            if (timeout > 0) {
                subsys.sendAck(Duration.ofSeconds(timeout));
            } else {
                subsys.sendAck(null);
            }
        } else {
            subsys.sendNack("Sending NACK as requested.");
            return "NACK";
        }

        if (delayAfterAck > 0) {
            try {
                Thread.sleep(delayAfterAck * 1000L);
            } catch (InterruptedException x) {
                return "Interrupted while waiting to send the result";
            }
        }
        return "Done.";
    }

    @Command(type = CommandType.ACTION, description = "Publish AIDA plot")
    public void publishPlot(String key, String path, String title) {
        try {
            KeyValueDataList plots = new KeyValueDataList("test_plots");
            IHistogram1D histObj = histogramFactory.createHistogram1D(path, title, 50, -3, 6);
            for (int i = 0; i < 500; i++) {
                histObj.fill(r.nextGaussian());
            }
            plots.addData(key, XMLUtils.createXMLString((IManagedObject) histObj), KeyValueData.KeyValueDataType.KeyValuePlotData);
            subsys.publishSubsystemDataOnStatusBus(plots);
        } catch (IOException ioe) {
        }
    }
    
    @Command(type = CommandType.ACTION)
    public void updateState(String componentName, DemoState state) {
        if("".equals(componentName)) {
            subsys.updateAgentState(state);
        } else {
            subsys.updateAgentComponentState(componentName, state);
        }
    }

}

==> org-lsst-ccs-subsystem-demo-main-5.1.7-sources.jar: org/lsst/ccs/subsystem/demo/main/DemoSubsystem.java <==
package org.lsst.ccs.subsystem.demo.main;

import java.util.concurrent.TimeUnit;
import org.lsst.ccs.bus.data.KeyValueData;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.subsystem.demo.bus.DemoData;
import org.lsst.ccs.utilities.scheduler.PeriodicTask;
import hep.aida.*;
import hep.aida.util.XMLUtils;
import java.io.IOException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.Random;
import org.lsst.ccs.Subsystem;
import org.lsst.ccs.bus.data.Alert;
import org.lsst.ccs.bus.data.KeyValueDataList;
import org.lsst.ccs.bus.states.AlertState;
import org.lsst.ccs.command.annotations.Command.CommandType;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.framework.AgentPeriodicTask;
import org.lsst.ccs.framework.HasLifecycle;
import org.lsst.ccs.services.AgentPeriodicTaskService;
import org.lsst.ccs.utilities.scheduler.Scheduler;

public class DemoSubsystem implements HasLifecycle {
    
    public static enum DemoState {
        IDLE, BUSY;
    }
    
    public static enum OtherState {
        LOW, HIGH;
    }
    
    public static enum AState {
        A,B;
    }

    /**
     * Scheduler task that publishes heartbeat status messages.
     */
    protected PeriodicTask dataPublisher;
    protected PeriodicTask plotPublisher;

    //Create analysis factory
    private final IAnalysisFactory af = IAnalysisFactory.create();
    // Create datapoint set factory
    private final IDataPointSetFactory dataPointSetFactory = af.createDataPointSetFactory(null);
    // Create histogram factory
    private final IHistogramFactory histogramFactory = af.createHistogramFactory(null);

    private final Random r = new Random();

    private final ArrayList<PeriodicTask> tracerTestTasks = new ArrayList<>();

    @LookupField(strategy = LookupField.Strategy.TREE)
    protected AgentPeriodicTaskService periodicTaskService;

    @LookupField(strategy = LookupField.Strategy.TOP)
    protected Subsystem subsys;
    
    private int dataPublishRate = 5;
    private int dataSize = 2;
    
    /**
     ***************************************************************************
     **
     ** Initializes the subsystem. *
     * **************************************************************************
     */
    @Override
    public void postStart() {
        // This must not be invoked in init because the subsystem is not yet 
        // connected on the buses.
        updateLimits("1.0", "2.0");
    }

    @Override
    public void build() {

        //Schedule a periodic task to publish demo data
        periodicTaskService.scheduleAgentPeriodicTask(
                new AgentPeriodicTask("demoData-publish",
                        () -> {
                            DemoData dd = new DemoData(dataSize);
                            KeyValueData d = new KeyValueData("demo_Data", dd);
                            KeyValueDataList kvdl = new KeyValueDataList();
                            kvdl.addData(d);
                            subsys.publishSubsystemDataOnStatusBus(kvdl);
                        }).withIsFixedRate(false).withPeriod(Duration.ofSeconds(dataPublishRate)));
        
        //Schedule a periodic task to publish demo plots
        periodicTaskService.scheduleAgentPeriodicTask(
                new AgentPeriodicTask("demoPlots-publish",
                        () -> {
                            try {
                                
                                KeyValueDataList plots = new KeyValueDataList("");
                                IHistogram1D histObj = histogramFactory.createHistogram1D("test 1d", 50, -3, 6);
                                for (int i = 0; i < 500; i++) {
                                    histObj.fill(r.nextGaussian());
                                }
                                
                                IDataPointSet dps1D = dataPointSetFactory.create("dps1D", "one dimensional IDataPointSet", 1);
                                for (int i = 0; i < 10; i++) {
                                    dps1D.addPoint().coordinate(0).setValue(r.nextDouble());
                                }
                                
                                plots.addData("histogram1D", XMLUtils.createXMLString((IManagedObject) histObj), KeyValueData.KeyValueDataType.KeyValuePlotData);
                                plots.addData("dataPointSet", XMLUtils.createXMLString((IManagedObject) dps1D), KeyValueData.KeyValueDataType.KeyValuePlotData);
                                
                                subsys.publishSubsystemDataOnStatusBus(plots);
                            } catch (IOException ioe) {
                                ioe.printStackTrace();
                            }
                        }).withIsFixedRate(false).withPeriod(Duration.ofSeconds(5)));        
        
    }
    
    @Command(type = CommandType.ACTION, description = "update limits")
    public void updateLimits(String low, String high) {
        KeyValueDataList limits = new KeyValueDataList("demo_Data/temp");
        limits.addData("alarmLow", low, KeyValueData.KeyValueDataType.KeyValueMetaData);
        limits.addData("alarmHigh", high, KeyValueData.KeyValueDataType.KeyValueMetaData);
        subsys.publishSubsystemDataOnStatusBus(limits);
    }

    @Command(type = CommandType.QUERY, description = "raise custom alert")
    public void raiseCustomAlert(String alertId, AlertState severity, String cause) {
        Alert a = new Alert(alertId, "Custom Alert");
        subsys.getAlertService().raiseAlert(a, severity, cause);
    }
    
    @Command(type = CommandType.QUERY, description = "Get Subsystem Data Sample")
    public DemoData getSampleData() {
        return new DemoData(dataSize);
    }

    @Command(type = CommandType.QUERY, description = "Set Subsystem Data Sample")
    public void setSampleDataSize(int size) {
        this.dataSize = size;
    }
    
    
    @Command(type = CommandType.QUERY, description = "Get Subsystem Array Sample")
    public Object[] getSampleArrayData() {
        Object[] data = new Object[]{"test", 2.3, new int[]{3, 2, 2, 2}};
        return data;
    }
    
    @Command(type = CommandType.QUERY, description = "Throw an Exception")
    public void throwAnException() {
        throw new RuntimeException("An Exception");
    }

    @Command(type = CommandType.ACTION, description = "Enable/Disable Publishing Tracer Test Messages")
    public void runTracerTest(boolean enable) {
        if (enable) {
            Scheduler s = subsys.getScheduler();
            tracerTestTasks.add(s.scheduleAtFixedRate(() -> subsys.getLogger().info("Self test on module " + r.nextInt(1000) + (r.nextInt(5) > 0 ? ": OK" : ": FAILED")), 0, 7, TimeUnit.SECONDS));
            tracerTestTasks.add(s.scheduleAtFixedRate(() -> subsys.getLogger().warn("London Bridge is in trouble, please repair."), 2, 15, TimeUnit.SECONDS));
            tracerTestTasks.add(s.scheduleAtFixedRate(() -> subsys.getLogger().error("Sun has gone supernova."), 3, 13, TimeUnit.SECONDS));
        } else {
            tracerTestTasks.forEach(task -> task.cancel(false));
            tracerTestTasks.clear();
        }
    }

    @Command(type = CommandType.QUERY, description = "Time out")
    public String testTimeout(int sleep) {
        if (sleep > 0) {
            try {
                Thread.sleep(sleep * 1000L);
            } catch (InterruptedException x) {
                return "Interrupted";
            }
        }
        return "Done.";
    }

    @Command(type = CommandType.QUERY, description = "Test sending ACK/NACK. Delays are in seconds.", autoAck = false)
    public String testAck(boolean accept, int delayBeforeAck, int delayAfterAck, int timeout) {

        if (delayBeforeAck > 0) {
            try {
                Thread.sleep(delayBeforeAck * 1000L);
            } catch (InterruptedException x) {
                return "Interrupted while waiting to send ACK/NACK";
            }
        }

        if (accept) {
            if (timeout > 0) {
                subsys.sendAck(Duration.ofSeconds(timeout));
            } else {
                subsys.sendAck(null);
            }
        } else {
            subsys.sendNack("Sending NACK as requested.");
            return "NACK";
        }

        if (delayAfterAck > 0) {
            try {
                Thread.sleep(delayAfterAck * 1000L);
            } catch (InterruptedException x) {
                return "Interrupted while waiting to send the result";
            }
        }
        return "Done.";
    }

    @Command(type = CommandType.ACTION, description = "Publish AIDA plot")
    public void publishPlot(String key, String path, String title) {
        try {
            KeyValueDataList plots = new KeyValueDataList("test_plots");
            IHistogram1D histObj = histogramFactory.createHistogram1D(path, title, 50, -3, 6);
            for (int i = 0; i < 500; i++) {
                histObj.fill(r.nextGaussian());
            }
            plots.addData(key, XMLUtils.createXMLString((IManagedObject) histObj), KeyValueData.KeyValueDataType.KeyValuePlotData);
            subsys.publishSubsystemDataOnStatusBus(plots);
        } catch (IOException ioe) {
        }
    }
    
    @Command(type = CommandType.ACTION)
    public void updateState(String componentName, DemoState state) {
        if("".equals(componentName)) {
            subsys.updateAgentState(state);
        } else {
            Object componentToUpdate = subsys.getComponentLookup().getComponentByPath(componentName);
            if ( componentToUpdate == null ) {
                throw new IllegalArgumentException("No component corresponds to path "+componentName);
            }
            subsys.updateAgentComponentState(componentToUpdate, state);
        }
    }

}
